import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  map,
  merge,
  Observable,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { User } from '../../user/models/user';

import { PartnerRecommendation as PartnerRecommendation } from '../models/partner-recommendation';
import { PartnerRecommendationPagination as PartnerRecommendationPagination } from '../models/partner-recommendation-pagination';
import { PartnerRecommendationService as PartnerRecommendationService } from '../services/partner-recommendation.service';

import { Router } from '@angular/router';

import { MatDialog } from '@angular/material/dialog';
import { RegistComposeComponent } from '../compose/regist-compose.component';
@Component({
  selector: 'partner-recommendation-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .inventory-grid {
        /*                     CB   상부   아이디 등급 계좌정보 */
        grid-template-columns: 20px 140px auto 100px 200px;

        @screen sm {
          /*                     CB   상부  아이디 등급 계좌정보  머니  사이트  사용여부*/
          grid-template-columns: 20px 140px auto 100px 200px 140px 200px 140px;
        }

        @screen md {
          /*                     CB   상부  아이디 등급 계좌정보  머니  사이트  사용여부  회원수 */
          grid-template-columns: 20px 140px auto 100px 200px 140px 200px 140px 140px;
        }

        @screen lg {
          /*                     CB   상부  아이디 등급 계좌정보  머니  사이트  사용여부 회원수 */
          grid-template-columns: 20px 100px auto 100px 200px 140px 200px 140px 140px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  partnerRecommendations$!: Observable<PartnerRecommendation[] | undefined>;
  users$!: Observable<User[] | undefined>;

  __isSearchOpened1 = false;
  __isSearchOpened2 = false;
  isLoading = false;
  searchInputControl = new FormControl();
  selectedPartnerRecommendation?: PartnerRecommendation;
  pagination?: PartnerRecommendationPagination;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _partnerRecommendationService: PartnerRecommendationService,
    private router: Router,
    private _matDialog: MatDialog
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._partnerRecommendationService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: PartnerRecommendationPagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products
    this.partnerRecommendations$ =
      this._partnerRecommendationService.partnerRecommendations$;
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'name',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.markForCheck();

      // If the partnerRecommendation changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;
            return this._partnerRecommendationService.getPartnerRecommendations(
              this._paginator.pageIndex,
              this._paginator.pageSize,
              this._sort.active,
              this._sort.direction
            );
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    }
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  viewUserDetail(id: string): void {
    let url: string = 'member/user/' + id;
    this.router.navigateByUrl(url);
  }
  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * toggle the search
   * Used in 'bar'
   */
  __onClickSearch1(): void {
    this.__isSearchOpened1 = !this.__isSearchOpened1;
  }
  __onClickSearch2(): void {
    this.__isSearchOpened2 = !this.__isSearchOpened2;
  }

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __onClickRegist(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(RegistComposeComponent);

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
}
