import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  map,
  merge,
  Observable,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { User } from '../../../member/user/models/user';
import { TodayBet } from '../models/today-bet';
import { TodayBetPagination } from '../models/today-bet-pagination';
import { TodayBetService } from '../services/today-bet.service';
import { Router } from '@angular/router';
import { MatTableDataSource } from '@angular/material/table';

@Component({
  selector: 'today-bet-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .inventory-grid {
        grid-template-columns: 60px auto 40px;

        @screen sm {
          grid-template-columns: 60px 60px 60px 60px 60px 60px auto 60px;
        }

        @screen md {
          grid-template-columns: 60px 60px 60px 60px 60px 60px auto 60px 60px;
        }

        @screen lg {
          grid-template-columns: 60px 70px 70px 70px 70px 100px 60px 60px auto 60px 60px 60px 60px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  todayBets$!: Observable<TodayBet[] | undefined>;
  users$!: Observable<User[] | undefined>;

  isLoading = false;
  searchInputControl = new FormControl();
  selectedTodayBet?: TodayBet;

  todayBetForm!: FormGroup;

  todayBetDataSource: MatTableDataSource<any> = new MatTableDataSource();
  todayBetTableColumns: string[] = [
    'rank',
    'level',
    'signinId',
    'nickname',
    'depositDetails',
    'depositPartnerDetails',
    'totalProfit',
    'passiveMoney',
    'passiveComp',
    'casinoBetDetails',
    'casinoWinningDetails',
    'casinoWinLoss',
    'casinoCommission',
    'casinoSettle',
    'slotBetDetails',
    'slotWinningDetails',
    'slotWinLoss',
    'slotCommission',
    'slotSettle',
    'powerballBetDetails',
    'powerballWinningDetails',
    'powerballWinLoss',
    'powerballCommission',
    'powerballSettle',
    'totalBetWinSettle',
  ];
  private _unsubscribeAll: Subject<any> = new Subject<any>();

  roles1: any[];
  roles2: any[];

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _todayBetService: TodayBetService,
    private router: Router
  ) {
    this.roles1 = [
      {
        label: '전체',
        value: '전체',
      },
      {
        label: '파워볼',
        value: '파워볼',
      },
      {
        label: '카지노',
        value: '카지노',
      },
      {
        label: '슬롯',
        value: '슬롯',
      },
    ];
    this.roles2 = [
      {
        label: '회원아이디',
        value: '회원아이디',
      },
      {
        label: '닉네임',
        value: '닉네임',
      },
    ];
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    this.todayBetForm = this._formBuilder.group({
      bankName: [''],
      accountNumber: [''],
      accountHolder: [''],
    });
    // Get the pagination
    this._todayBetService
      .getTodayBets()
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((todayBets: any) => {
        // Mark for check
        this.todayBetDataSource = todayBets.todayBets;
        console.log(todayBets);
        this._changeDetectorRef.markForCheck();
      });

    // Get the products
    this.todayBets$ = this._todayBetService.todayBets$;
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {}

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  viewUserDetail(id: string): void {
    let url: string = 'member/user/' + id;
    this.router.navigateByUrl(url);
  }
  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }
  __testData(info: any): any {
    console.log(info);
  }
}
